# -*- coding: utf-8 -*-

from zope.event import notify
from zope.lifecycleevent import Attributes
from zope.component import getGlobalSiteManager
from zope.interface import implementedBy, providedBy
from dolmen.forms.crud import ObjectInitializedEvent
from megrok.z3cform.base.utils import set_fields_data


def notify_object_creation(fields, content, data):
    """Builds a list of descriptions, made of Attributes objects, defining
    the changes made on the content and the related interface.
    """
    changes = set_fields_data(fields, content, data)
    if changes:
        descriptions = []
        for interface, names in changes.items():
            descriptions.append(Attributes(interface, *names))
        notify(ObjectInitializedEvent(content, *descriptions))
        return descriptions
    return None


def queryClassMultiAdapter(adapts, ob, interface, name=u''):
    """This function searches the component registry for any adapter
    registered for an instance of the given class. It lookups and returns
    it correctly factored.
    """
    sm = getGlobalSiteManager()
    klass = adapts[0]
    required = implementedBy(klass)
    lookfor = (required,) + tuple(providedBy(a) for a in adapts[1:])
    factory = sm.adapters.lookup(lookfor, interface, name)
    if factory is not None:
        return factory(ob, *adapts[1:])
    return None



from interfaces import IAdding, IFactoryAdding
from interfaces import IFieldsCustomization, IObjectInitializedEvent
from events import ObjectInitializedEvent
from addview import Adder
from crudforms import Display, Add, Edit, Delete
from customize import FieldsCustomizer



# -*- coding: utf-8 -*-

import megrok.pagetemplate as pt
from dolmen.forms.crud import Display


class DisplayTemplate(pt.PageTemplate):
    """The basic template for a display form.
    """
    pt.view(Display)



# -*- coding: utf-8 -*-

import grokcore.component as grok

from dolmen.content import IBaseContent
from dolmen.forms.base import IFieldUpdate
from dolmen.forms.crud import IObjectInitializedEvent

from zope.component import getAdapters
from zope.lifecycleevent import ObjectModifiedEvent, ObjectCreatedEvent


class ObjectInitializedEvent(ObjectCreatedEvent):
    """An object has been created and initialized with form values.
    """
    grok.implements(IObjectInitializedEvent)
    
    def __init__(self, object, *descriptions) :
        super(ObjectInitializedEvent, self).__init__(object)
        self.descriptions = descriptions


@grok.subscribe(IBaseContent, ObjectModifiedEvent)
@grok.subscribe(IBaseContent, ObjectInitializedEvent)
def notify_fields_update(ob, event):
    """This handler propagates the ObjectModifiedEvent to a more atomic
    level, by calling an adapter on each modified field. This permits to
    actually interact at the field level, after it gets modified.
    """
    for desc in event.descriptions:
        for name in desc.attributes:
            field = desc.interface[name]
            handlers = getAdapters((ob, field), IFieldUpdate)
            for handler in handlers:
                # Iteration through the generator
                pass



# -*- coding: utf-8 -*-

import dolmen.content as content
import dolmen.forms.base as form
import grokcore.component as grok

from zope.component import queryMultiAdapter
from zope.i18nmessageid import MessageFactory
from zope.cachedescriptors.property import CachedProperty
from dolmen.forms.crud import utils, interfaces as crud

_ = MessageFactory("dolmen")


class Add(form.PageAddForm):
    """The add form itself is not protected. The security is checked on
    'update'. It checks if the 'require' directive of the factored item
    is respected on the context.
    """
    grok.baseclass()
    grok.title(_(u"Add"))
    grok.name('dolmen.add')
    grok.context(crud.IFactoryAdding)

    form_name = _(u"Add")

    @property
    def label(self):
        return self.context.factory.title

    @CachedProperty
    def fields(self):
        ifaces = self.context.factory.getSchema()
        fields = form.Fields(*ifaces).omit('__parent__')
        
        modifier = utils.queryClassMultiAdapter(
            (self.context.factory.factory, self, self.request),
            self.context,
            crud.IFieldsCustomization
            )

        if modifier is not None:
            return modifier(fields)
        return fields

    def nextURL(self):
        return self.context.nextURL()

    def createAndAdd(self, data):
        obj = self.create(data)
        self.add(obj)
        return obj

    def add(self, object):
        return self.context.add(object)

    def create(self, data):
        obj = self.context.factory()
        utils.notify_object_creation(self.fields, obj, data)
        return obj

    @form.button.buttonAndHandler(_('Save'), name='save')
    def handleSave(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        obj = self.createAndAdd(data)
        if obj is not None:
            self.redirect(self.url(obj))


class Edit(form.PageEditForm):
    grok.baseclass()
    grok.title(_(u"Edit"))
    grok.context(content.IBaseContent)
    form.extends(form.PageEditForm, ignoreButtons=True)
    
    form_name = _(u"Edit")

    @property
    def label(self):
        return _(u"edit_action", default=u"Edit: $name",
                 mapping={"name": self.context.title})

    def nextURL(self):
        return self.url(self.context)

    @CachedProperty
    def fields(self):
        iface = content.schema.bind().get(self.context)
        fields = form.Fields(*iface).omit('__parent__')
        modifier = queryMultiAdapter(
            (self.context, self, self.request),
            crud.IFieldsCustomization
            )
        if modifier is not None:
            return modifier(fields)
        return fields

    @form.button.buttonAndHandler(_('Apply'), name='apply')
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        changes = form.apply_data_event(self.fields, self.context, data)
        if changes:
            self.status = self.successMessage
        else:
            self.status = self.noChangesMessage
        self.redirect(self.nextURL())


class Display(form.PageDisplayForm):
    grok.baseclass()
    grok.title(_(u"View"))
    grok.context(content.IBaseContent)

    ignoreContext = False

    @property
    def label(self):
        return self.context.title

    @CachedProperty
    def fields(self):
        iface = content.schema.bind().get(self.context)
        fields = form.Fields(*iface).omit('__parent__', 'title')
        modifier = queryMultiAdapter(
            (self.context, self, self.request),
            crud.IFieldsCustomization
            )
        if modifier is not None:
            return modifier(fields)
        return fields


class Delete(form.PageForm):
    """A confirmation for to delete an object.
    """
    grok.baseclass()
    grok.title(_(u"Delete"))
    grok.context(content.IBaseContent)

    label = _(u"Confirm deletion")
    form_name = _(u"Are you really sure ?")
    fields = {}

    _deleted = False
    
    def delete(self):
        container = self.context.__parent__
        name = self.context.__name__
        if name in container:
            try:
                del container[name]
                self._deleted = True
                return self._deleted
            except ValueError, e:
                pass
        self._deleted = False
        return self._deleted

    def nextURL(self):
        if self._deleted == False:
            return self.url(self.context)
        self.redirect(self.url(self.context))

    @form.button.buttonAndHandler(_('Confirm'), name='confirm')
    def handleConfirm(self, action):
        result = self.delete()
        if result is False:
            self.status = _("This object could not be deleted")
        self.redirect(self.nextURL())



from os.path import join
from setuptools import setup, find_packages

name = 'dolmen.forms.crud'
version = '0.2'
readme = open(join('src', 'dolmen', 'forms', 'crud', 'README.txt')).read()
history = open(join('docs', 'HISTORY.txt')).read()

install_requires = [
    'setuptools',
    'grokcore.security',
    'grokcore.component',
    'grokcore.viewlet',
    'zope.event',
    'zope.schema',
    'zope.location',
    'zope.security',
    'zope.component',
    'zope.interface',
    'zope.publisher',
    'zope.traversing',
    'zope.lifecycleevent',
    'zope.cachedescriptors',
    'zope.app.container',
    'dolmen.field >= 0.3',
    'dolmen.content >= 0.2.2',
    'dolmen.forms.base >= 0.1',
    'megrok.pagetemplate >= 0.3',
    'megrok.z3cform.base >= 0.1',
    ]

tests_require = install_requires + [
    'zope.testing',
    'zope.securitypolicy',
    'zope.app.testing',
    'zope.app.zcmlfiles',
    ]

setup(name = name,
      version = version,
      description = "Auto-generated forms for dolmen.content",
      long_description = readme + '\n\n' + history,
      keywords = 'Dolmen Grok Content Forms',
      author = 'Souheil Chelfouh',
      author_email = 'trollfot@gmail.com',
      url = '',
      license = 'GPL',
      packages = find_packages('src', exclude=['ez_setup']),
      package_dir = {'': 'src'},
      namespace_packages = ['dolmen', 'dolmen.forms'],
      include_package_data = True,
      zip_safe = False,
      tests_require = tests_require,
      install_requires = install_requires,
      extras_require = {'test': tests_require},
      test_suite = "dolmen.forms.crud",
      classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Zope3',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
          ],
      )



# -*- coding: utf-8 -*-

import re
import os.path
import unittest

from zope.testing import doctest
from zope.app.testing import functional

ftesting_zcml = os.path.join(os.path.dirname(__file__), 'ftesting.zcml')
FunctionalLayer = functional.ZCMLLayer(
    ftesting_zcml, __name__, 'FunctionalLayer', allow_teardown=True
    )

def test_suite():
    suite = unittest.TestSuite()
    readme = functional.FunctionalDocFileSuite(
        '../README.txt',
        globs={'__name__': 'dolmen.forms.crud.tests'}
        )
    readme.layer = FunctionalLayer
    suite.addTest(readme)
    return suite
